@@ -359,6 +359,7 @@ odbc_bcp_bind(TDS_DBC *dbc, const void * varaddr, int prefixlen, int varlen,
 	 const void * terminator, int termlen, int vartype, int table_column)
 {
 	TDSCOLUMN *colinfo;
+	TDSICONV *conv;
 
 	tdsdump_log(TDS_DBG_FUNC, "bcp_bind(%p, %p, %d, %d -- %p, %d, %d, %d)\n",
 						dbc, varaddr, prefixlen, varlen,
@@ -408,6 +409,16 @@ odbc_bcp_bind(TDS_DBC *dbc, const void * varaddr, int prefixlen, int varlen,
 	colinfo->column_bindlen  = varlen;
 	colinfo->bcp_prefix_len = prefixlen;
 
+	conv = colinfo->char_conv;
+	if (conv) {
+		TDSCONNECTION *conn = dbc->tds_socket->conn;
+		TDS_SERVER_TYPE desttype = tds_get_conversion_type(colinfo->on_server.column_type, colinfo->on_server.column_size);
+		TDS_SERVER_TYPE srctype = colinfo->column_bindtype == 0 ? desttype : (TDS_SERVER_TYPE) colinfo->column_bindtype;
+		int src_charset = is_unicode_type(srctype) ? odbc_get_wide_canonic(conn) : dbc->original_charset_num;
+
+		colinfo->char_conv = tds_iconv_get_info(conn, src_charset, conv->to.charset.canonic);
+	}
+
 	TDS_ZERO_FREE(colinfo->bcp_terminator);
 	colinfo->bcp_term_len = 0;
 	if (termlen > 0) {
